package com.wechat.common.util.rsa;

import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.cert.X509Certificate;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

public class RsaUtil {
	
	/**
	 * RSA敏感信息加密
	 * @param message
	 * @param certificate
	 * @return
	 * @throws IllegalBlockSizeException
	 * @throws IOException
	 */
	public static String rsaEncryptOAEP(String message, X509Certificate certificate)
			throws IllegalBlockSizeException, IOException {
		  try {
		    Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPWithSHA-1AndMGF1Padding");
		    cipher.init(Cipher.ENCRYPT_MODE, certificate.getPublicKey());

		    byte[] data = message.getBytes("utf-8");
		    byte[] cipherdata = cipher.doFinal(data);
		    return Base64.getEncoder().encodeToString(cipherdata);
		  } catch (NoSuchAlgorithmException | NoSuchPaddingException e) {
		    throw new RuntimeException("当前Java环境不支持RSA v1.5/OAEP", e);
		  } catch (InvalidKeyException e) {
		    throw new IllegalArgumentException("无效的证书", e);
		  } catch (IllegalBlockSizeException | BadPaddingException e) {
		    throw new IllegalBlockSizeException("加密原串的长度不能超过214字节");
		  }
	}
	
	/**
	 * RAS敏感信息解密
	 * @param ciphertext
	 * @param privateKey
	 * @return
	 * @throws BadPaddingException
	 * @throws IOException
	 */
	public static String rsaDecryptOAEP(String ciphertext, PrivateKey privateKey)
		      throws BadPaddingException, IOException {
		    try {
		      Cipher cipher = Cipher.getInstance("RSA/ECB/OAEPWithSHA-1AndMGF1Padding");
		      cipher.init(Cipher.DECRYPT_MODE, privateKey);

		      byte[] data = Base64.getDecoder().decode(ciphertext);
		      return new String(cipher.doFinal(data), "utf-8");
		    } catch (NoSuchPaddingException | NoSuchAlgorithmException e) {
		      throw new RuntimeException("当前Java环境不支持RSA v1.5/OAEP", e);
		    } catch (InvalidKeyException e) {
		      throw new IllegalArgumentException("无效的私钥", e);
		    } catch (BadPaddingException | IllegalBlockSizeException e) {
		      throw new BadPaddingException("解密失败");
		    }
	}
	
	/**
	 * 构造签名的方法
	 * @param data	构造签名的数据
	 * @param priKey  商户私钥
	 * @return
	 * @throws Exception
	 */
	public static byte[] signRSA(String data, String priKey) throws Exception {
		//签名的类型
		Signature sign = Signature.getInstance("SHA256withRSA");
		//读取商户私钥,该方法传入商户私钥证书的内容即可
		byte[] keyBytes = org.apache.commons.codec.binary.Base64.decodeBase64(priKey);
		PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);
		KeyFactory keyFactory = KeyFactory.getInstance("RSA");
		PrivateKey privateKey = keyFactory.generatePrivate(keySpec);
		sign.initSign(privateKey);
		sign.update(data.getBytes("UTF-8"));
		return sign.sign();
	}
	
	
	
}
