package com.wechat.core.ftp;

import com.wechat.common.util.StringUtil;
import com.wechat.config.FtpConfig;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * ftp 通用工具
 * @author actor
 * @date 2018/11/14 15:20
 */
public class FtpUtils {
    //ftp服务器地址
    private String hostName;
    //ftp服务器端口号默认为21
    private int port;
    //ftp登录账号
    private String userName;
    //ftp登录密码
    private String password;

    private FTPClient ftpClient = null;

    public FTPClient getFtpClient() {
        return ftpClient;
    }

    public FtpUtils(String hostName, int port, String userName, String password){
        this.hostName = hostName;
        this.port = port;
        this.userName = userName;
        this.password = password;
        if(!initFtpClient())throw new RuntimeException("初始化ftp失败");
    }
    public FtpUtils(FtpConfig ftpConfig){
        this.hostName = ftpConfig.getHost();
        this.port = ftpConfig.getPort();
        this.userName = ftpConfig.getUsername();
        this.password = ftpConfig.getPassword();
        if(!initFtpClient())throw new RuntimeException("初始化ftp失败");
    }
    /**
     * 初始化ftp服务器
     */
    public boolean initFtpClient() {
        boolean bl = false;
        ftpClient = new FTPClient();
        try {
            ftpClient.connect(hostName, port); //连接ftp服务器
            ftpClient.login(userName, password); //登录ftp服务器
            int replyCode = ftpClient.getReplyCode(); //是否成功登录服务器
            if(!FTPReply.isPositiveCompletion(replyCode)){
                System.out.println("获取的code:"+replyCode);
                ftpClient.disconnect();
                return false;
            } else bl = true;
            ftpClient.enterLocalPassiveMode();
            ftpClient.setControlEncoding("utf-8");// 中文支持
            ftpClient.setBufferSize(1024);
            ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);//设置文件类型
        }catch (MalformedURLException e) {
            e.printStackTrace();
        }catch (IOException e) {
            e.printStackTrace();
        }
        return bl;
    }

    /**
     * 本地  -- >   上传文件
     * @param pathname ftp服务保存地址
     * @param fileName 上传到ftp的文件名
     *  @param originfilename 待上传文件的名称（绝对地址） *
     * @return
     */
    public boolean uploadFile( String pathname, String fileName,String originfilename){
        try {
            return  uploadFile(pathname,fileName,new FileInputStream(new File(originfilename)));
        }catch (Exception e){
            e.printStackTrace();
        }
        return false;
    }
    /**
     * 本地--->  上传文件
     * @param pathname ftp服务保存地址
     * @param fileName 上传到ftp的文件名
     * @param inputStream 输入文件流
     * @return
     */
    public boolean uploadFile( String pathname, String fileName,InputStream inputStream){
        boolean flag = false;
        try{
            //创建远程目录
            _createDirecroty(ftpClient,pathname);
            ftpClient.changeWorkingDirectory("/");// 换到根目录上
            if(pathname.length() > 0 )
                ftpClient.changeWorkingDirectory(pathname);
            ftpClient.storeFile(fileName, inputStream);
            flag = true;
        }catch (Exception e) {
            e.printStackTrace();
        }finally{
            if(null != inputStream){
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return flag;
    }

    /**
     * 创建多层目录文件，如果有ftp服务器已存在该文件，则不创建，如果无，则创建
     * @param pathName 远程路径
     * @return
     * @throws IOException
     * @date 创建时间：2017年6月22日 上午11:51:33
     */
    public static boolean _createDirecroty(FTPClient ftpClient,String pathName)
            throws IOException {
        if(StringUtil.isEmpty(pathName) || pathName.equalsIgnoreCase("/")) return true;//如果为空 不用创建目录
        if(!pathName.endsWith("/")) pathName +="/";
        pathName = new String(pathName.getBytes("UTF-8"),"iso-8859-1");//支持中文
        String tmpPath ="";
        if (!ftpClient.makeDirectory(new String(pathName))) {
            int start;
            int end;
            if (pathName.startsWith("/")) start = 1;
            else start = 0;
            end = pathName.indexOf("/", start);
            while (true) {
                String subDirectory = pathName.substring(start, end);
                tmpPath +="/"+subDirectory;
                if (!isExistDir(tmpPath, ftpClient)) {
                    if (ftpClient.makeDirectory(subDirectory)) ftpClient.changeWorkingDirectory(subDirectory);
                     else return false;
                } else ftpClient.changeWorkingDirectory(subDirectory);
                start = end + 1;
                end = pathName.indexOf("/", start);
                if (end <= start)  break;
            }
        }
        ftpClient.changeWorkingDirectory("/");
        return true;
    }
    /**
     * 判断ftp服务器文件是否存在
     * @param path
     * @param ftp
     * @return
     * @throws IOException
     * @date 创建时间：2017年6月22日 上午11:52:52
     */
    public static boolean isExistFile(String path, FTPClient ftp) throws IOException {
        return ftp.listFiles(path).length > 0;
    }
    /**
     * 判断ftp服务器目录是否存在
     * @param path
     * @param ftp
     * @return
     * @throws IOException
     * @date 创建时间：2017年6月22日 上午11:52:52
     */
    public static boolean isExistDir(String path, FTPClient ftp) throws IOException {
        if(!path.startsWith("/")) path = "/"+path;
        return ftp.changeWorkingDirectory(path);
    }

    /** *本地--> 下载文件 *
     * @param pathname FTP服务器文件目录 *
     * @param fileName 文件名称 *
     * @param localPath 下载后的文件路径 *
     * @return */
    public  boolean downloadFile(String pathname, String fileName, String localPath){
        boolean flag = false;
        OutputStream os = null;
        try {
            //切换FTP目录
            if(pathname.length() > 0 ) ftpClient.changeWorkingDirectory(pathname);
            for(FTPFile file : ftpClient.listFiles()){
                if(file.isFile() && fileName.equalsIgnoreCase(file.getName())){//是文件并且相等就下载
                    os = new FileOutputStream( new File(localPath + File.separatorChar + file.getName()));
                    ftpClient.retrieveFile(file.getName(), os);
                    flag = true;
                    os.close();
                    break;
                }
            }
            ftpClient.logout();
        } catch (Exception e) {
            e.printStackTrace();
        } finally{
            if(ftpClient.isConnected()){
                try{
                    ftpClient.disconnect();
                }catch(IOException e){
                    e.printStackTrace();
                }
            }
            if(null != os){
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return flag;
    }
    public Byte[] downloadFileStrem(String pathName, String fileName){
        InputStream in = null;
        try {
            in = getFtpInputStream(pathName, fileName);
            List<Byte> list = new ArrayList<Byte>();
            int n = -1;
            while((n = in.read()) != -1){
                list.add((byte) n);
            }
            int size = list.size();
            /*byte[] by = new byte[size];
            for(int i=0;i<size;i++){
                by[i] = list.get(i);
            }*/
            return list.toArray(new Byte[size]);
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            if(in != null){
                try {
                    in.close();
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        }
        return null;
    }
    public byte[] downloadFileStrem2(String pathName, String fileName){
        InputStream in = null;
        try {
            in = getFtpInputStream(pathName, fileName);
            List<Byte> list = new ArrayList<Byte>();
            int n = -1;
            while((n = in.read()) != -1){
                list.add((byte) n);
            }
            int size = list.size();
            byte[] by = new byte[size];
            for(int i=0;i<size;i++){
                by[i] = list.get(i);
            }
            return by;
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            if(in != null){
                try {
                    in.close();
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        }
        return null;
    }
    public byte[] downloadFileByOutStrean(String pathName,String fileName){
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        if (getFtpOutputStream(pathName,fileName,out)) {
           return out.toByteArray();
        }
        return null;
    }

    private boolean getFtpOutputStream(String pathName, String fileName,OutputStream out) {
        try {
            ftpClient.changeWorkingDirectory("/");
            if(StringUtil.isNotEmpty(pathName)) ftpClient.changeWorkingDirectory(pathName);
            return ftpClient.retrieveFile(fileName,out);
        }catch (Exception e){
            e.printStackTrace();
        }
        return false;
    }

    /**获取ftp的输入流*/
    public InputStream getFtpInputStream(String pathName, String fileName){
        try {
            ftpClient.changeWorkingDirectory("/");
            if(StringUtil.isNotEmpty(pathName)) ftpClient.changeWorkingDirectory(pathName);
            return ftpClient.retrieveFileStream(fileName);
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 远程下载
     * @param response
     * @param pathname 远程目录名
     * @param fileName 远程文件名
     * @return
     */
    public boolean downloadFileINPC(HttpServletResponse response, String pathname, String fileName){
        boolean flag = false;
        OutputStream os = null;
        try{
            //切换FTP目录
            if(pathname.length() > 0 )
                ftpClient.changeWorkingDirectory(pathname);
            for(FTPFile file : ftpClient.listFiles()){
                if(file.isFile() && fileName.equalsIgnoreCase(file.getName())){//是文件并且相等就下载
                    os = response.getOutputStream();
                    ftpClient.retrieveFile(file.getName(), os);
                    flag = true;
                    os.close();
                    break;
                }
            }
            ftpClient.logout();
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            if(ftpClient.isConnected()){
                try{
                    ftpClient.disconnect();
                }catch(IOException e){
                    e.printStackTrace();
                }
            }
            if(null != os){
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return flag;
    }

    /** *  删除文件 *
     * @param pathname FTP服务器保存目录 *
     * @param filename 要删除的文件名称 *
     * @return */
    public boolean deleteFile(String pathname, String filename){
        boolean flag = false;
        try {
            ftpClient.changeWorkingDirectory("/");
            //切换FTP目录
            if(pathname.length() > 0)
                ftpClient.changeWorkingDirectory(pathname);
            flag = ftpClient.deleteFile(filename);
            ftpClient.logout();
            return flag;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if(ftpClient.isConnected()){
                try{
                    ftpClient.disconnect();
                }catch(IOException e){
                    e.printStackTrace();
                }
            }
        }
        return flag;
    }

    /***
     * 内部删除
     * @param ftpClient
     * @param fileName
     * @return
     */
    private boolean _deleteFile(FTPClient ftpClient,String fileName){
        boolean flag = false;
        try {
            ftpClient.changeWorkingDirectory("/");
            flag = ftpClient.deleteFile(fileName);
        }catch (Exception e){e.printStackTrace();}
        return flag;
    }

    /***
     * 批量删除目录与文件 逐步删除 并关闭
     * @param ftpClient
     * @param pathName
     * @return
     */
    public boolean deleteDirAndFileBatch(FTPClient ftpClient,String pathName){
        try {
            String[] split = pathName.split(",");
            List<String> collect = Arrays.stream(split).filter(s -> s.trim().length() > 0).distinct().collect(Collectors.toList());
            int size = collect.size();
            int num = 0;
            System.out.println();
            for (int i=0;i<size;i++){
                if(_deleteDirAndFile(ftpClient,collect.get(i))) num++;
            }
            if(num == size) return true;
        }catch (Exception e){
            e.printStackTrace();
        }finally {
            try {
                ftpClient.logout();
            }catch (Exception e){}
            try {
                ftpClient.disconnect();
            }catch (Exception e){}
        }
        return false;
    }
    /**
     * 删除文件与目录 多级删除
     * @param ftpClient
     * @param pathname
     * @return
     */
    public boolean deleteDirAndFile(FTPClient ftpClient,String pathname){
        try {
            return _deleteDirAndFile(ftpClient,pathname);
        }catch (Exception e){e.printStackTrace();}finally {
            try {
                ftpClient.logout();
            }catch (Exception e){}
            try {
                ftpClient.disconnect();
            }catch (Exception e){}
        }
        return false;
    }
    /**
     * 删除文件与目录 多级删除
     * @param ftpClient
     * @param pathname
     * @return
     */
    private boolean _deleteDirAndFile(FTPClient ftpClient,String pathname){
        try {
            ftpClient.changeWorkingDirectory("/");
            if(!ftpClient.removeDirectory(pathname)) {
                ftpClient.changeWorkingDirectory(pathname);
                //下面使用一段一段的删除目录
                FTPFile[] ftpFiles = ftpClient.listFiles();
                for (FTPFile file : ftpFiles) {
                    if (file.isFile()) if (!_deleteFile(ftpClient,pathname+"/"+ file.getName())) return false;
                    if (file.isDirectory()) {//目录删除
                        if (!_deleteDirAndFile(ftpClient, pathname + "/" + file.getName())) return false;
                    }
                }
                //移除子目录后继续删除
                ftpClient.changeWorkingDirectory("/");
                return ftpClient.removeDirectory(pathname);
            }
            return true;
        }catch (Exception e){e.printStackTrace();}
        return false;
    }

    /**
     * https://www.comtikaiot.com/v3/upload_ftp/
     * 193.112.52.171
     * 18012
     * comtika_ftp
     * comtika2018
     * @param args
     * @throws Exception
     */

    public static void main(String[] args)throws Exception {
        FtpUtils ftpUtils = new FtpUtils("www.comtikaiot.com",18012,"comtika_ftp","comtika2018");
        String path = "/";
        String fileName = "test2.txt";
        //ftpUtils.downloadFile(path,fileName,"F:/");
        System.out.println(ftpUtils.uploadFile(path,fileName,"G:/dy.png"));
        ftpUtils.close();

    }

    public void close() {
        try {
            if(ftpClient != null) ftpClient.logout();
        }catch (Exception e){}finally {
            if(ftpClient.isConnected()){
                try{
                    ftpClient.disconnect();
                }catch(IOException e){
                    e.printStackTrace();
                }
            }
        };
    }
}
